import java.net.MalformedURLException;
import java.rmi.AlreadyBoundException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.rmi.server.UnicastRemoteObject;
import java.util.*;


/**
 * Implementacao do cliente
 * 
 */
public class ClientImpl extends UnicastRemoteObject implements Client {
	
	protected Setinha setinha;
	protected String id;
	protected Hashtable setinhasMap;
	protected SlaveServer slave;
	
	protected Janela janela;
	
	protected SetinhaCopia minhaSetinha;

	/**
	 * Contrutor, dado um ID para ele, e um servidor principal, ele pede ao servidor principal por um
	 * slave, e ai conecta-se neste slave e ganha uma setinha para ele
	 */
	public ClientImpl(String id, MasterServer master) throws RemoteException {
		super();
		this.id = id;
		slave = master.getServer();
		System.out.println("Registrando no servidor " + slave.getId());		
		this.setinha = slave.newSetinha(this);	
		
		janela = new Janela(this);
		setinhasMap = new Hashtable();
	}
	
	public /*synchronized*/ void avisa(Setinha s){ 
	
		//throws RemoteException {
		try{
			
			String sId = s.getId();
			
			System.out.println("Setinha " + sId + " se mexeu.");
			
			SetinhaCopia sc = new SetinhaCopia(s.getX(), s.getY(), s.getAngle() );
			
			setinhasMap.put(sId, sc);
			
			janela.myPaint(setinhasMap);		
		}
		catch(Exception e){
			System.out.println("Erro recebendo aviso de setinha/Lendo atributos da seta " 
								+ e);
		}
	
	}

	public String getId(){
		return id;
	}
	
	public Setinha getSetinha() {
		return setinha;	
	}
	
	public SetinhaCopia getMinhaSetinha(){
		return minhaSetinha;
	}
	
    public void exit() throws RemoteException{
	slave.disconnect(this);
    }
    
	
	public static void main(String args[]) throws Exception {

		// Create and install a security manager
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new RMISecurityManager());
		}

		MasterServer master = (MasterServer) Naming.lookup("//"+args[0]+"/MasterServer");
		
		ClientImpl cliente = new ClientImpl(args[1], master);
		
		
		cliente.getSetinha().setX(100);
		cliente.getSetinha().setY(100);
		cliente.getSetinha().setAngle(90);
		cliente.slave.notifyMasterServer(cliente.getSetinha());
		
		cliente.minhaSetinha = new SetinhaCopia(100, 100, 90);
		
	
	}
	
		
}
