import java.net.MalformedURLException;
import java.rmi.AlreadyBoundException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.rmi.server.UnicastRemoteObject;

//import java.util.ArrayList;
//import java.util.Iterator;
import java.util.*;

/**
 * Implementacao do servidor principal
 * 
 */
public class MasterServerImpl extends UnicastRemoteObject implements MasterServer {
	/**
	 * ArrayList com todos os servidores slaves que esse master pode utilizar
	 */
	//protected ArrayList slaves = new ArrayList();
	protected Vector slaves = new Vector();
	
    protected ServerLoadBalancer lb = new ServerRRLoadBalancer();

	/**
	 * Constutor que recebe uma lista de servidores slaves que ele podera usar. Os servidores slaves DEVEM estar 
	 * ligados antes do mestre ser ligado.
	 */
	public MasterServerImpl(String [] lista) throws RemoteException, MalformedURLException, NotBoundException{
	
		for(int i = 0; i < lista.length; i+=2){
			SlaveServer slave = (SlaveServer) Naming.lookup("//" + lista[i+1] + "/SlaveServer" + lista[i]);
			slaves.add(slave);
		} 	
	}
	
	public synchronized SlaveServer getServer() {
		 //throws RemoteException {
		try{
			return lb.balanceLoad(slaves.iterator());
		}
		catch(Exception e){
			System.out.println("Nao foi possivel dar um SlaveServer pro cliente " + e);
		}
		return null;
	}
	
    //only notify diferente slaves
	public synchronized void notifySlaveServers(Setinha s) {
	  	//throws RemoteException {
		try{
			Iterator iterator = slaves.iterator();
			while(iterator.hasNext()){
		  		((SlaveServer)iterator.next()).notifyClients(s);
			}	
		}	
		catch(Exception e){
			System.out.println("Erro notificando slave servers." + e);
		}
	}

	/**
	 * Simplesmente cria um novo servidor mestre e conecta-o aos slaves
	 */	
	public static void main (String [] args) throws RemoteException, MalformedURLException, NotBoundException, NotBoundException{
	
		MasterServerImpl masterServer = new MasterServerImpl(args);
		
		Naming.rebind("//localhost/MasterServer", masterServer);
		
		System.out.println("master server criado");
	}

	
	
}
