import java.rmi.Remote;
import java.rmi.RemoteException;

public interface SlaveServer extends Remote {


	/**
	 * Cria uma setinha para determinado cliente, e registra ela neste servidor
	 * O slave eh o responsavel por criar uma setinha, pois, se nao fosse assim, o cliente nao poderia registrar 
	 * a setinha em outra maquina, ja que a politica do Naming.bind() nao permite isso.
	 *implementacao deve ser synchronized  
	 */
	Setinha newSetinha(Client c) throws RemoteException; 
	
    //o master chama este metodo quando algum cliente C de outro slave mudou
    // notifica os clientes desse server, que o cliente C mudou
	void notifyClients(Setinha s)  throws RemoteException ;
	
    //clientes chamam este metodo quando suas setinhas mudam
    // notifica o servidor principal, para que ele notifique os slaves e tal
    //implementacao deve ser synchronized  
    void notifyMasterServer(Setinha s)  throws RemoteException ;
	
	/**
	 * Pega o load deste servidor. o load pode ser implementado de modos
	 *diferentes.
	 * politica de load balancing
	 */
	Load getLoad() throws RemoteException;
	
	/**
	 * Devolve o ID deste slave, pois podem haver mais de um slave na mesma maquina, o que traria confusao.
	 */
	String getId() throws RemoteException;

    //metodo que disconecta um cliente
    void disconnect(Client c) throws RemoteException;
}
	




